@@ -9,7 +9,7 @@ use crate::serialize::{Deserialize, Serialize};
 use crate::value::{built_in_function::BuiltInFunction, ApplyMulHandling, Value};
 use crate::Attrs;
 use std::sync::Arc;
-use std::{fmt, io};
+use std::{borrow, cmp, fmt, io};
 
 #[derive(Clone, Copy, Debug, PartialEq, Eq)]
 pub(crate) enum BitwiseBop {
@@ -99,7 +99,7 @@ impl fmt::Display for Bop {
 	}
 }
 
-#[derive(Clone, Debug, PartialEq, Eq)]
+#[derive(Clone, Debug)]
 pub(crate) enum Expr {
 	Literal(Value),
 	Ident(Ident),
@@ -127,6 +127,43 @@ pub(crate) enum Expr {
 }
 
 impl Expr {
+	pub(crate) fn compare<I: Interrupt>(&self, other: &Self, int: &I) -> FResult<bool> {
+		Ok(match (self, other) {
+			(Self::Literal(a), Self::Literal(b)) => {
+				a.compare(b, int)? == Some(cmp::Ordering::Equal)
+			}
+			(Self::Ident(a), Self::Ident(b)) => a == b,
+			(Self::Parens(a), Self::Parens(b)) => a.compare(b, int)?,
+			(Self::UnaryMinus(a), Self::UnaryMinus(b)) => a.compare(b, int)?,
+			(Self::UnaryPlus(a), Self::UnaryPlus(b)) => a.compare(b, int)?,
+			(Self::UnaryDiv(a), Self::UnaryDiv(b)) => a.compare(b, int)?,
+			(Self::Factorial(a), Self::Factorial(b)) => a.compare(b, int)?,
+			(Self::Bop(a1, a2, a3), Self::Bop(b1, b2, b3)) => {
+				a1 == b1 && a2.compare(b2, int)? && a3.compare(b3, int)?
+			}
+			(Self::Apply(a1, a2), Self::Apply(b1, b2)) => {
+				a1.compare(b1, int)? && a2.compare(b2, int)?
+			}
+			(Self::ApplyFunctionCall(a1, a2), Self::ApplyFunctionCall(b1, b2)) => {
+				a1.compare(b1, int)? && a2.compare(b2, int)?
+			}
+			(Self::ApplyMul(a1, a2), Self::ApplyMul(b1, b2)) => {
+				a1.compare(b1, int)? && a2.compare(b2, int)?
+			}
+			(Self::As(a1, a2), Self::As(b1, b2)) => a1.compare(b1, int)? && a2.compare(b2, int)?,
+			(Self::Fn(a1, a2), Self::Fn(b1, b2)) => a1 == b1 && a2.compare(b2, int)?,
+			(Self::Of(a1, a2), Self::Of(b1, b2)) => a1 == b1 && a2.compare(b2, int)?,
+			(Self::Assign(a1, a2), Self::Assign(b1, b2)) => a1 == b1 && a2.compare(b2, int)?,
+			(Self::Equality(a1, a2, a3), Self::Equality(b1, b2, b3)) => {
+				a1 == b1 && a2.compare(b2, int)? && a3.compare(b3, int)?
+			}
+			(Self::Statements(a1, a2), Self::Statements(b1, b2)) => {
+				a1.compare(b1, int)? && a2.compare(b2, int)?
+			}
+			_ => false,
+		})
+	}
+
 	pub(crate) fn serialize(&self, write: &mut impl io::Write) -> FResult<()> {
 		match self {
 			Self::Literal(x) => {
@@ -382,7 +419,7 @@ pub(crate) fn evaluate<I: Interrupt>(
 			let a = eval!(*a)?;
 			match a {
 				Value::Num(a) => Value::Num(Box::new(a.sub(eval!(*b)?.expect_num()?, int)?)),
-				Value::Date(a) => a.sub(eval!(*b)?)?,
+				Value::Date(a) => a.sub(eval!(*b)?, int)?,
 				f @ (Value::BuiltInFunction(_) | Value::Fn(_, _, _)) => f.apply(
 					Expr::UnaryMinus(b),
 					ApplyMulHandling::OnlyApply,
@@ -473,8 +510,10 @@ pub(crate) fn evaluate<I: Interrupt>(
 		Expr::Equality(is_equals, a, b) => {
 			let lhs = evaluate(*a, scope.clone(), attrs, context, int)?;
 			let rhs = evaluate(*b, scope.clone(), attrs, context, int)?;
-
-			Value::Bool(if is_equals { lhs == rhs } else { lhs != rhs })
+			Value::Bool(match lhs.compare(&rhs, int)? {
+				Some(cmp::Ordering::Equal) => is_equals,
+				Some(cmp::Ordering::Greater | cmp::Ordering::Less) | None => !is_equals,
+			})
 		}
 	})
 }
@@ -516,7 +555,7 @@ fn evaluate_add<I: Interrupt>(
 			)),
 			scope,
 		),
-		(Value::Date(d), b) => d.add(b)?,
+		(Value::Date(d), b) => d.add(b, int)?,
 		_ => return Err(FendError::ExpectedANumber),
 	})
 }
@@ -533,7 +572,7 @@ fn evaluate_as<I: Interrupt>(
 		match ident.as_str() {
 			"bool" | "boolean" => {
 				let num = evaluate(a, scope, attrs, context, int)?.expect_num()?;
-				return Ok(Value::Bool(!num.is_zero()));
+				return Ok(Value::Bool(!num.is_zero(int)?));
 			}
 			"date" => {
 				let a = evaluate(a, scope, attrs, context, int)?;
@@ -582,6 +621,15 @@ fn evaluate_as<I: Interrupt>(
 				}
 				return Err(FendError::ExpectedANumber);
 			}
+			"roman" | "roman_numeral" => {
+				let a = evaluate(a, scope, attrs, context, int)?
+					.expect_num()?
+					.try_as_usize(int)?;
+				if a == 0 {
+					return Err(FendError::RomanNumeralZero);
+				}
+				return Ok(Value::String(borrow::Cow::Owned(to_roman(a))));
+			}
 			_ => (),
 		}
 	}
@@ -715,3 +763,30 @@ pub(crate) fn resolve_identifier<I: Interrupt>(
 		_ => return crate::units::query_unit(ident.as_str(), attrs, context, int),
 	})
 }
+
+fn to_roman(mut num: usize) -> String {
+	// based on https://stackoverflow.com/a/41358305
+	let mut result = String::new();
+	for (r, n) in [
+		("M", 1000),
+		("CM", 900),
+		("D", 500),
+		("CD", 400),
+		("C", 100),
+		("XC", 90),
+		("L", 50),
+		("XL", 40),
+		("X", 10),
+		("IX", 9),
+		("V", 5),
+		("IV", 4),
+		("I", 1),
+	] {
+		let q = num / n;
+		num -= q * n;
+		for _ in 0..q {
+			result.push_str(r);
+		}
+	}
+	result
+}
