@@ -539,11 +539,14 @@ fn parse_symbol(ch: char, input: &mut &str) -> FResult<Token> {
 				Symbol::Equals
 			}
 		}
+		'\u{2260}' => Symbol::NotEquals,       // unicode not equal to symbol
 		'\\' | '\u{3bb}' => Symbol::Backslash, // lambda symbol
 		'.' => Symbol::Dot,
 		'<' => {
 			if test_next('<') {
 				Symbol::ShiftLeft
+			} else if test_next('>') {
+				Symbol::NotEquals
 			} else {
 				return Err(FendError::UnexpectedChar(ch));
 			}
