@@ -3171,6 +3171,16 @@ fn acos_minus_one_point_zero_one() {
 	test_eval("acos (-1.01)", "approx. 3.1415926535 - 0.1413037694i");
 }
 
+#[test]
+fn acos_one() {
+	test_eval("acos 1", "approx. 0");
+}
+
+#[test]
+fn acos_minus_one() {
+	test_eval("acos (-1)", "approx. 3.1415926535");
+}
+
 #[test]
 fn atan_1() {
 	test_eval("atan 1", "approx. 0.7853981633");
@@ -5878,11 +5888,51 @@ fn test_superscript() {
 fn test_equality() {
 	test_eval("1 + 2 == 3", "true");
 	test_eval("1 + 2 != 4", "true");
+	test_eval("1 + 2 ≠ 4", "true");
+	test_eval("1 + 2 <> 4", "true");
 	test_eval("true == false", "false");
 	test_eval("true != false", "true");
+	test_eval("true ≠ false", "true");
 	test_eval("2m == 200cm", "true");
 	test_eval("2kg == 200cm", "false");
 	test_eval("2kg == true", "false");
 	test_eval("2.010m == 200cm", "false");
 	test_eval("2.000m == approx. 200cm", "true");
 }
+
+#[test]
+fn test_roman() {
+	expect_error(
+		"0 to roman",
+		Some("zero cannot be represented as a roman numeral"),
+	);
+	test_eval_simple("1 to roman", "I");
+	test_eval_simple("2 to roman", "II");
+	test_eval_simple("3 to roman", "III");
+	test_eval_simple("4 to roman", "IV");
+	test_eval_simple("5 to roman", "V");
+	test_eval_simple("6 to roman", "VI");
+	test_eval_simple("7 to roman", "VII");
+	test_eval_simple("8 to roman", "VIII");
+	test_eval_simple("9 to roman", "IX");
+	test_eval_simple("10 to roman", "X");
+	test_eval_simple("11 to roman", "XI");
+	test_eval_simple("12 to roman", "XII");
+	test_eval_simple("13 to roman", "XIII");
+	test_eval_simple("14 to roman", "XIV");
+	test_eval_simple("15 to roman", "XV");
+	test_eval_simple("16 to roman", "XVI");
+	test_eval_simple("17 to roman", "XVII");
+	test_eval_simple("18 to roman", "XVIII");
+	test_eval_simple("19 to roman", "XIX");
+	test_eval_simple("20 to roman", "XX");
+	test_eval_simple("21 to roman", "XXI");
+	test_eval_simple("22 to roman", "XXII");
+	test_eval_simple("45 to roman", "XLV");
+	test_eval_simple("134 to roman", "CXXXIV");
+	test_eval_simple("1965 to roman", "MCMLXV");
+	test_eval_simple("2020 to roman", "MMXX");
+	test_eval_simple("3456 to roman", "MMMCDLVI");
+	test_eval_simple("1452 to roman", "MCDLII");
+	test_eval_simple("20002 to roman", "MMMMMMMMMMMMMMMMMMMMII");
+}
